@@ -62,14 +62,6 @@
 !
 !    - SUBOURTINE HETMIXLIN
 !    - function FUNCPA
-!    - subroutine MARQ9
-!    - subroutine DERIVA
-!    - subroutine SEARPAS
-!    - subroutine VALFPA
-!    - subroutine MAXT
-!    - subroutine DCHOLE
-!    - subroutine DMFSD
-!    - subroutine DSINV
 !    - subroutine POSTPROB
 !
 !----------------------------------------------------------
@@ -94,894 +86,6 @@
 
 !*************************************************************
 
-
-
-
-
-
-
-      module parameters
-          double precision,save::epsa,epsb,epsd
-          integer,save::maxiter
-      end module parameters
-
-
-
-! module pour la verification de type argument
-      module type
-
-      interface verif1
-      subroutine marq98(b,m,ni,v,rl,ier,istop,ca,cb,dd)
-         integer,intent(in) :: m
-         integer,intent(inout)::ni,ier,istop
-         double precision,dimension(m*(m+3)/2),intent(out)::v
-         double precision,intent(out)::rl
-         double precision,dimension(m),intent(inout)::b
-	 double precision,intent(inout)::ca,cb,dd
-      end subroutine marq98
-
-      subroutine deriva(b,m,v,rl)
-        integer,intent(in)::m
-        double precision,intent(inout)::rl
-        double precision,dimension(m),intent(in)::b
-        double precision,dimension((m*(m+3)/2)),intent(out)::v
-      end subroutine deriva
-
-      subroutine searpas(vw,step,b,bh,m,delta,fim)
-        integer,intent(in)::m
-        double precision,dimension(m),intent(in)::b
-        double precision,dimension(m),intent(inout)::bh,delta
-        double precision,intent(inout)::vw,fim,step
-      end subroutine searpas
-
-      subroutine dmfsd(a,n,eps,ier)
-        integer,intent(in)::n
-        integer,intent(inout)::ier
-        double precision,intent(inout)::eps
-        double precision,dimension(n*(n+1)/2),intent(inout)::A
-      end subroutine dmfsd
-
-      subroutine valfpa(vw,fi,b,bk,m,delta)
-        integer,intent(in)::m
-	double precision,intent(in)::vw
-        double precision,dimension(m),intent(in)::b,delta
-        double precision,dimension(m),intent(out)::bk
-        double precision,intent(out)::fi
-      end subroutine valfpa
-
-
-      end interface verif1
-
-      interface verif2
-      subroutine dsinv(A,N,EPS,IER,DET)
-        integer,intent(in)::n
-        integer,intent(inout)::ier
-        double precision,intent(inout)::eps
-        double precision,intent(inout),optional::det
-        double precision,dimension(n*(n+1)/2),intent(inout)::A
-      end subroutine dsinv
-
-      subroutine dchole(a,k,nq,idpos)
-      integer,intent(in)::k,nq
-      integer,intent(inout)::idpos
-      double precision,dimension(k*(k+3)/2),intent(inout)::a
-      end subroutine dchole
-
-      subroutine maxt(maxtt,delta,m)
-
-      double precision,intent(out)::maxtt
-       integer,intent(in)::m
-       double precision,dimension(m),intent(in)::delta
-       end subroutine maxt
-      end interface verif2
-
-
-      end module type
-
-
-!-----------------------------------------------------------
-! Derniere mis a jour : 09/02/2011
-!c-----------------------------------------------------------
-!c*************************************************************
-
-
-!C-------------------------------------------------------------
-!C                  MODULE OPTIM
-!
-!
-!C-------------------------------------------------------------
-      module optim
-
-      implicit none
-!c -Interface permettant la verification des type des arguments
-      interface verif1
-        module procedure marq98,deriva,searpas,dmfsd,valfpa
-      end interface verif1
-
-      interface verif2
-        module procedure dsinv,dchole,maxt
-      end interface verif2
-
-
-      CONTAINS
-!C-------------------------------------------------------------
-!C                   MARQ98
-!C-------------------------------------------------------------
-
-
-      subroutine marq98(b,m,ni,v,rl,ier,istop,ca,cb,dd)
-
-!c
-!c  fu = matrice des derivees secondes et premieres
-!c
-!c  istop: raison de l'arret
-!c  1: critere d'arret satisfait (prm=ca, vraisblce=cb, derivee=dd)
-!c  2: nb max d'iterations atteints
-!c  4: Erreur
-
-      use parameters
-
-
-
-      IMPLICIT NONE
-
-
-!C   variables globales
-      integer,intent(in) :: m
-      integer,intent(inout)::ni,ier,istop
-      double precision,dimension(m*(m+3)/2),intent(out)::v
-      double precision,intent(out)::rl
-      double precision,dimension(m),intent(inout)::b
-      double precision,intent(inout)::ca,cb,dd
-
-!C   variables locales
-      integer::nql,ii,nfmax,idpos,ncount,id,jd,m1,j,i,ij
-      double precision,dimension(m*(m+3)/2)::fu
-      double precision,dimension(m)::delta,b1,bh
-      double precision::da,dm,ga,tr
-      double precision::GHG,funcpa,det,step,eps  &
-         ,vw,fi,z,rl1,th,ep
-
-      double precision::maxtt
-!C ----------------------------------------------------
-
-      id=0
-      jd=0
-      z=0.d0
-      th=1.d-5
-      eps=1.d-7!1.d-6
-      nfmax=m*(m+1)/2
-      ca=epsa+1.d0
-      cb=epsb+1.d0
-      rl1=-1.d+10
-      ni=0
-      istop=0
-      da=0.01d0
-      dm=5.d0
-      nql=1
-      m1=m*(m+1)/2
-      ep=1.d-20
-      Main:Do
-        call deriva(b,m,v,rl)
-        rl1=rl
-
-        dd = 0.d0
-!c        do i=m*(m+1)/2+1,m*(m+3)/2
-!c           dd = dd + v(i)*v(i)
-!c        end do
-!c        dd=dd/dabs(RL)
-
-        fu=0.D0
-        do i=1,m
-           do j=i,m
-              ij=(j-1)*j/2+i
-              fu(ij)=v(ij)
-           end do
-        end do
-
-        call dsinv(fu,m,ep,ier,det)
-
-        if (ier.eq.-1) then
-           dd=epsd+1.d0
-        else
-           GHG = 0.d0
-           do i=1,m
-              do j=1,m
-                 if(j.ge.i) then
-                    ij=(j-1)*j/2+i
-                 else
-                    ij=(i-1)*i/2+j
-                 end if
-                 GHG = GHG + v(m1+i)*fu(ij)*V(m1+j)
-	      end do
-           end do
-           dd=GHG/dble(m)
-        end if
-
-        if(ca.lt.epsa.and.cb.lt.epsb.and.dd.lt.epsd) exit main
-
-        tr=0.d0
-        do i=1,m
-           ii=i*(i+1)/2
-           tr=tr+dabs(v(ii))
-        end do
-        tr=tr/dble(m)
-
-        ncount=0
-        ga=0.01d0
- 400    do i=1,nfmax+m
-           fu(i)=v(i)
-        end do
-        do i=1,m
-           ii=i*(i+1)/2
-           if (v(ii).ne.0) then
-              fu(ii)=v(ii)+da*((1.d0-ga)*dabs(v(ii))+ga*tr)
-           else
-              fu(ii)=da*ga*tr
-           endif
-        end do
-        call dchole(fu,m,nql,idpos)
-
-        if (idpos.ne.0) then
-           ncount=ncount+1
-           if (ncount.le.3.or.ga.ge.1.d0) then
-              da=da*dm
-           else
-              ga=ga*dm
-              if (ga.gt.1.d0) ga=1.d0
-           endif
-           goto 400
-        else
-            do i=1,m
-               delta(i)=fu(nfmax+i)
-               b1(i)=b(i)+delta(i)
-            end do
-            rl=funcpa(b1,m,id,z,jd,z)
-            if (rl1.lt.rl) then
-               if(da.lt.eps) then
-                  da=eps
-               else
-                  da=da/(dm+2.d0)
-               endif
-               goto 800
-            endif
-         endif
-!C      write(6,*) 'loglikelihood not improved '
-         call maxt(maxtt,delta,m)
-         if(maxtt.eq.0.D0) then
-            vw=th
-         else
-            call maxt(maxtt,delta,m)
-            vw=th/maxtt
-         endif
-         step=dlog(1.5d0)
-!C      write(*,*) 'searpas'
-         call searpas(vw,step,b,bh,m,delta,fi)
-         rl=-fi
-         if(rl.eq.-1.D9) then
-               istop=4
-               goto 110
-          end if
-
-         do i=1,m
-            delta(i)=vw*delta(i)
-         end do
-         da=(dm-3.d0)*da
-
- 800     cb=dabs(rl1-rl)
-         ca=0.d0
-         do i=1,m
-            ca=ca+delta(i)*delta(i)
-         end do
-!C         write(6,*) 'ca =',ca,' cb =',cb,' dd =',dd
-         do i=1,m
-            b(i)=b(i)+delta(i)
-         end do
-
-         ni=ni+1
-         if (ni.ge.maxiter) then
-            istop=2
-!C            write(6,*) 'maximum number of iteration reached'
-            goto 110
-         end if
-      End do Main
-
-
-       v=0.D0	 
-       v(1:m*(m+1)/2)=fu(1:m*(m+1)/2)
-
-
-      istop=1
-
- 110   continue
-       return
-       end subroutine marq98
-
-
-
-!C------------------------------------------------------------
-!C                          DERIVA
-!C------------------------------------------------------------
-
-      subroutine deriva(b,m,v,rl)
-
-      implicit none
-
-      integer,intent(in)::m
-      double precision,intent(inout)::rl
-      double precision,dimension(m),intent(in)::b
-      double precision,dimension((m*(m+3)/2)),intent(out)::v
-      double precision,dimension(m)::fcith
-      integer ::i0,m1,ll,i,k,j
-      double precision::funcpa,thn,th,z,vl,temp,thi,thj
-!c
-!c     v:matrice d'information+score
-!c     calcul de la derivee premiere par "central difference"
-!c     calcul des derivees secondes par "forward difference"
-!c
-
-      z=0.d0
-      i0=0
-
-      rl=funcpa(b,m,i0,z,i0,z)
-
-      if(rl.eq.-1.d9) then
-         goto 123
-      end if
-
-      do i=1,m
-         th=DMAX1(1.d-7, 1.d-4 * DABS(b(i)))
-         fcith(i)=funcpa(b,m,i,th,i0,z)
-         if(fcith(i).eq.-1.d9) then
-            rl=-1.d9
-            goto 123
-         end if
-      end do
-
-      k=0
-      m1=m*(m+1)/2
-      ll=m1
-      Main:do i=1,m
-         ll=ll+1
-         thn=-DMAX1(1.d-7, 1.d-4 * DABS(b(i)))
-         temp=funcpa( b,m, i,thn,i0,z)
-         if(temp.eq.-1.d9) then
-            rl=-1.d9
-            exit Main
-         end if
-         vl=(fcith(i)-temp)/(2.d0*(-thn))
-         v(ll)=vl
-         do j=1,i
-            k=k+1
-
-            thi=DMAX1(1.d-7, 1.d-4 * DABS(b(i)))
-            thj=DMAX1(1.d-7, 1.d-4 * DABS(b(j)))
-
-            temp=funcpa(b,m,i,thi,j,thj)
-            if(temp.eq.-1.d9) then
-               rl=-1.d9
-               exit Main
-            end if
-            v(k)=-(temp-fcith(j)-fcith(i)+rl)   &
-           /(thi*thj)
-         end do
-      end do Main
- 123   continue
-
-      return
-      end subroutine deriva
-!C------------------------------------------------------------
-!C                        SEARPAS
-!C------------------------------------------------------------
-
-
-      subroutine searpas(vw,step,b,bh,m,delta,fim)
-!C
-!C  MINIMISATION UNIDIMENSIONNELLE
-!C
-      implicit none
-
-      integer,intent(in)::m
-      double precision,dimension(m),intent(in)::b
-      double precision,intent(inout)::vw
-      double precision,dimension(m),intent(inout)::bh,delta
-      double precision,intent(inout)::fim,step
-      double precision::vlw,vlw1,vlw2,vlw3,vm,fi1,fi2,fi3
-      integer::i
-
-
-       vlw1=dlog(vw)
-       vlw2=vlw1+step
-       call valfpa(vlw1,fi1,b,bh,m,delta)
-       call valfpa(vlw2,fi2,b,bh,m,delta)
-
-       if(fi2.ge.fi1) then
-	  vlw3=vlw2
-	  vlw2=vlw1
-	  fi3=fi2
-	  fi2=fi1
-	  step=-step
-
-          vlw1=vlw2+step
-          call valfpa(vlw1,fi1,b,bh,m,delta)
-          if(fi1.gt.fi2) goto 50
-       else
-          vlw=vlw1
-          vlw1=vlw2
-          vlw2=vlw
-          fim=fi1
-          fi1=fi2
-          fi2=fim
-       end if
-
-       do i=1,40
-          vlw3=vlw2
-          vlw2=vlw1
-          fi3=fi2
-          fi2=fi1
-
-          vlw1=vlw2+step
-          call valfpa(vlw1,fi1,b,bh,m,delta)
-          if(fi1.gt.fi2) goto 50
-          if(fi1.eq.fi2) then
-             fim=fi2
-             vm=vlw2
-             goto 100
-          end if
-       end do
-!C
-!C  PHASE 2 APPROXIMATION PAR QUADRIQUE
-!C
-50     continue
-!C
-!C  CALCUL MINIMUM QUADRIQUE
-!C
-      vm=vlw2-step*(fi1-fi3)/(2.d0*(fi1-2.d0*fi2+fi3))
-      call valfpa(vm,fim,b,bh,m,delta)
-      if(fim.le.fi2) goto 100
-      vm=vlw2
-      fim=fi2
-100   continue
-      vw=dexp(vm)
-
-      return
-
-      end subroutine searpas
-
-!C------------------------------------------------------------
-!C                         DCHOLE
-!C------------------------------------------------------------
-
-      subroutine dchole(a,k,nq,idpos)
-
-      implicit none
-
-      integer,intent(in)::k,nq
-      integer,intent(inout)::idpos
-      double precision,dimension(k*(k+3)/2),intent(inout)::a
-
-      integer::i,ii,i1,i2,i3,m,j,k2,jmk
-      integer::ijm,irm,jji,jjj,l,jj,iil,jjl,il
-      integer,dimension(k)::is
-      double precision ::term,xn,diag,p
-      equivalence (term,xn)
-
-
-!c      ss programme de resolution d'un systeme lineaire symetrique
-!c
-!c
-!c       k ordre du systeme /
-!c       nq nombre de seconds membres
-!c
-!c       en sortie les seconds membres sont remplaces par les solutions
-!c       correspondantes
-!c
-
-      i2=0
-      ii=0
-      idpos=0
-      k2=k+nq
-!c     calcul des elements de la matrice
-      do i=1,k
-         ii=i*(i+1)/2
-!c       elements diagonaux
-         diag=a(ii)
-         i1=ii-i
-         if(i-1.ne.0) goto 1
-         if(i-1.eq.0) goto 4
-1        i2=i-1
-         do l=1,i2
-             m=i1+l
-             p=a(m)
-             p=p*p
-             if(is(l).lt.0) goto 2
-             if(is(l).ge.0) goto 3
-2            p=-p
-3            diag=diag-p
-         end do
-
-4        if(diag.lt.0) goto 5
-         if(diag.eq.0) goto 50
-         if(diag.gt.0) goto 6
-5        is(i)=-1
-         idpos=idpos+1
-         diag=-dsqrt(-diag)
-         a(ii)=-diag
-         goto 7
-6        is(i)=1
-         diag=dsqrt(diag)
-         a(ii)=diag
-!c       elements non diagonaux
-7        i3=i+1
-         do j=i3,k2
-            jj=j*(j-1)/2+i
-            jmk=j-k-1
-            if(jmk.le.0) goto 9
-            if(jmk.gt.0) goto 8
-8           jj=jj-jmk*(jmk+1)/2
-9           term=a(jj)
-            if(i-1.ne.0) goto 10
-            if(i-1.eq.0) goto 13
-10          do l=1,i2
-               iil=ii-l
-               jjl=jj-l
-               p=a(iil)*a(jjl)
-               il=i-l
-               if(is(il).lt.0) goto 11
-               if(is(il).ge.0) goto 12
-11             p=-p
-12             term=term-p
-            end do
-13            a(jj)=term/diag
-	   end do
-      end do
-
-!c       calcul des solutions
-      jj=ii-k+1
-      do l=1,nq
-         jj=jj+k
-         i=k-1
-14       jji=jj+i
-         xn=a(jji)
-         if(i-k+1.lt.0) goto 20
-         if(i-k+1.ge.0) goto 22
-20       j=k-1
-21       jjj=jj+j
-         ijm=i+1+j*(j+1)/2
-         xn=xn-a(jjj)*a(ijm)
-         if(j-i-1.le.0) goto 22
-         if(j-i-1.gt.0) goto 30
-30       j=j-1
-         goto 21
-22       irm=(i+1)*(i+2)/2
-         a(jji)=xn/a(irm)
-         if(i.le.0) cycle
-         if(i.gt.0) goto 40
-40       i=i-1
-         go to 14
-      end do
-50    continue
-      return
-      end subroutine dchole
-
-
-      subroutine dmfsd(a,n,eps,ier)
-!C
-!C   FACTORISATION DE CHOLESKY D'UNE MATRICE SDP
-!C   MATRICE = TRANSPOSEE(T)*T
-!C   ENTREE : TABLEAU A CONTENANT LA PARTIE SUPERIEURE STOCKEE COLONNE
-!C            PAR COLONNE DE LA METRICE A FACTORISER
-!C   SORTIE : A CONTIENT LA PARTIE SUPPERIEURE DE LA MATRICE triangulaire T
-!C
-!C   SUBROUTINE APPELE PAR DSINV
-!C
-!C   N : DIM. MATRICE
-!C   EPS : SEUIL DE TOLERANCE
-!C   IER = 0 PAS D'ERREUR
-!C   IER = -1 ERREUR
-!C   IER = K COMPRIS ENTRE 1 ET N, WARNING, LE CALCUL CONTINUE
-!C
-      implicit none
-
-      integer,intent(in)::n
-      integer,intent(inout)::ier
-      double precision,intent(inout)::eps
-      double precision,dimension(n*(n+1)/2),intent(inout)::A
-      double precision :: dpiv,dsum,tol
-      integer::i,k,l,kpiv,ind,lend,lanf,lind
-
-!C
-!C   TEST ON WRONG INPUT PARAMETER N
-!C
-      dpiv=0.d0
-      if (n-1.lt.0) goto 12
-      if (n-1.ge.0) ier=0
-!C
-!C   INITIALIZE DIAGONAL-LOOP
-!C
-      kpiv=0
-      do k=1,n
-          kpiv=kpiv+k
-          ind=kpiv
-          lend=k-1
-!C
-!C   CALCULATE TOLERANCE
-!C
-          tol=dabs(eps*sngl(A(kpiv)))
-!C
-!C   START FACTORIZATION-LOOP OVER K-TH ROW
-!C
-         do i=k,n
-	    dsum=0.d0
-            if (lend.lt.0) goto 2
-            if (lend.eq.0) goto 4
-            if (lend.gt.0) goto 2
-!C
-!C   START INNER LOOP
-!C
-2           do l=1,lend
-               lanf=kpiv-l
-               lind=ind-l
-	       dsum=dsum+A(lanf)*A(lind)
-            end do
-
-!C
-!C   END OF INNEF LOOP
-!C
-!C   TRANSFORM ELEMENT A(IND)
-!C
-4           dsum=A(ind)-dsum
-            if (i-k.ne.0) goto 10
-            if (i-k.eq.0) goto 5
-!C
-!C   TEST FOR NEGATIVE PIVOT ELEMENT AND FOR LOSS OF SIGNIFICANCE
-!C
-5           if (sngl(dsum)-tol.le.0) goto 6
-            if (sngl(dsum)-tol.gt.0) goto 9
-6           if (dsum.le.0) goto 12  !c ici que ça plante  exit do
-            if (dsum.gt.0) goto 7
-7           if (ier.le.0) goto 8
-            if (ier.gt.0) goto 9
-8           ier=k-1
-!C
-!C   COMPUTE PIVOT ELEMENT
-!C
-9           dpiv=dsqrt(dsum)
-            A(kpiv)=dpiv
-            dpiv=1.D0/dpiv
-            goto 11
-!C
-!C   CALCULATE TERMS IN ROW
-!C
-10          A(ind)=dsum*dpiv
-11          ind=ind+i
-         end do
-	 ind=ind+i
-      end do
-
-!C
-!C   END OF DIAGONAL-LOOP
-!C
-      return
-12    ier=-1
-      return
-
-      end subroutine dmfsd
-
-
-!C------------------------------------------------------------
-!C                            DSINV
-!C------------------------------------------------------------
-
-
-      subroutine dsinv(A,N,EPS,IER,DET)
-
-!C
-!C     INVERSION D'UNE MATRICE SYMETRIQUE DEFINIE POSITIVE :
-!C
-!C     MATRICE = TRANSPOSEE(T)*T
-!C     INERSE(MATRICE) = INVERSE(T)*INVERSE(TRANSPOSEE(T))
-!C
-!C     A : TABLEAU CONTENANT LA PARTIE SUPERIEURE DE LA MATRICE A INVERSER
-!C         STOCKEE COLONNE PAR COLONNE
-!C     DIM. MATRICE A INVERSER = N
-!C     DIM. TABLEAU A = N*(N+1)/2
-!C
-!C     EPS : SEUIL DE TOLERANCE AU-DESSOUS DUQUEL UN PIVOT EST CONSIDERE
-!C           COMME NUL
-!C
-!C     IER : CODE D'ERREUR
-!C         IER=0 PAS D'ERREUR
-!C         IER=-1 ERREUR SUR LA DIM.N OU MATRICE PAS DEFINIE POSITIVE
-!C         IER=1 PERTE DE SIGNIFICANCE, LE CALCUL CONTINUE
-!C
-      implicit none
-
-      integer,intent(in)::n
-      integer,intent(inout)::ier
-      double precision,intent(inout)::eps
-      double precision,intent(inout),optional::det
-      double precision,dimension(n*(n+1)/2),intent(inout)::A
-      double precision::din,work
-      integer::ind,ipiv,i,j,k,l,min,kend,lhor,lver,lanf
-
-!C
-!C     FACTORIZE GIVEN MATRIX BY MEANS OF SUBROUTINE DMFSD
-!C     A=TRANSPOSE(T) * T
-!C
-
-      call dmfsd(A,n,eps,ier)
-
-
-
-      if (ier.lt.0) goto 9
-      if (ier.ge.0) det=0.d0
-!C
-!C     INVERT UPPER TRIANGULAR MATRIX T
-!C     PREPARE INVERSION-LOOP
-!C
-!C
-!CC calcul du log du determinant
-
-      do i=1,n
-         det=det+dlog(A(i*(i+1)/2))
-      end do
-      det=2*det
-      ipiv=n*(n+1)/2
-      ind=ipiv
-!C
-!C     INITIALIZE INVERSION-LOOP
-!C
-      do i=1,n
-         din=1.d0/A(ipiv)
-         A(ipiv)=din
-         min=n
-         kend=i-1
-         lanf=n-kend
-         if (kend.le.0) goto 5
-         if (kend.gt.0) j=ind
-!C
-!C     INITIALIZE ROW-LOOP
-!C
-         do k=1,kend
-	    work=0.d0
-	    min=min-1
-	    lhor=ipiv
-	    lver=j
-!C
-!C     START INNER LOOP
-!C
-            do l=lanf,min
-	        lver=lver+1
-		lhor=lhor+l
-                work=work+A(lver)*A(lhor)
-	    end do
-!C
-!C     END OF INNER LOOP
-!C
-            A(j)=-work*din
-            j=j-min
-	 end do
-
-!C
-!C     END OF ROW-LOOP
-!C
-5        ipiv=ipiv-min
-         ind=ind-1
-      end do
-
-!C
-!C     END OF INVERSION-LOOP
-!C
-!C     CALCULATE INVERSE(A) BY MEANS OF INVERSE(T)
-!C     INVERSE(A) = INVERSE(T) * TRANSPOSE(INVERSE(T))
-!C     INITIALIZE MULTIPLICATION-LOOP
-!C
-      do i=1,n
-         ipiv=ipiv+i
-	 j=ipiv
-!C
-!C     INITIALIZE ROW-LOOP
-!C
-	 do k=i,n
-	    work=0.d0
-	    lhor=j
-!C
-!C     START INNER LOOP
-!C
-            do l=k,n
-	        lver=lhor+k-i
-		work=work+A(lhor)*A(lver)
-   		lhor=lhor+l
-            end do
-!C
-!C     END OF INNER LOOP
-!C
-            A(j)=work
-            j=j+k
-	 end do
-         j=j+k
-      end do
-
-!C
-!C     END OF ROW-AND MULTIPLICATION-LOOP
-!C
-9     return
-      end subroutine dsinv
-
-!C------------------------------------------------------------
-!C                          VALFPA
-!C------------------------------------------------------------
-
-        subroutine valfpa(vw,fi,b,bk,m,delta)
-
-
-
-
-        implicit none
-
-        integer,intent(in)::m
-        double precision,dimension(m),intent(in)::b,delta
-        double precision,dimension(m),intent(out)::bk
-        double precision,intent(out)::fi
-	double precision::vw,funcpa,z
-	integer::i0,i
-
-         z=0.d0
-         i0=1
-         do i=1,m
-            bk(i)=b(i)+dexp(vw)*delta(i)
-	 end do
-         fi=-funcpa(bk,m,i0,z,i0,z)
-
-         return
-
-         end subroutine valfpa
-
-!C------------------------------------------------------------
-!C                            MAXT
-!C------------------------------------------------------------
-
-
-      subroutine maxt(maxtt,delta,m)
-
-      implicit none
-       double precision,intent(out) ::maxtt
-       integer,intent(in)::m
-       double precision,dimension(m),intent(in)::delta
-       integer::i
-!c
-       maxtt=Dabs(delta(1))
-       do i=2,m
-         if(Dabs(delta(i)).gt.maxtt)then
-	    maxtt=Dabs(delta(i))
-	 end if
-       end do
-
-       return
-       end subroutine maxt
-
-      end module optim
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 !================ SUBROUTINES ================================
 
       subroutine hetmixlin(Y0,X0,Prior0, idprob0,idea0,idg0   &
@@ -1026,6 +130,7 @@
       double precision, dimension(ns0*nea0), intent(out)::pred_RE
       double precision, dimension(nobs0) :: resid_m, resid_ss
       double precision, dimension(nobs0,ng0):: pred_m_g, pred_ss_g
+      double precision,external::funcpa
 
 
       maxmes=0
@@ -1175,7 +280,7 @@
          cb=0.d0
          dd=0.d0
 
-         call marq98(b,npm,ni,V,vrais,ier,istop,ca,cb,dd)
+         call marq98(b,npm,ni,V,vrais,ier,istop,ca,cb,dd,funcpa)
 
 !        write(*,*)
 !        write(*,*)'    FIN OPTIMISATION  ..... '
